% [X, dX, ddX,dddX]=HermiteSpline(P, ind) calcola
% l'interpolante i nodi P in forma di Hermite con
% parametrizzazione ind
%
% Input
%   P   nodi da interpolare
%   ind parametrizzazione
%       1 - uniforme
%       2 - cord-length
%       3 - centripeta
% Output
%   X    curva
%   dX   derivata prima
%   ddX  derivata seconda
%   dddX derivata terza

function [X, dX, ddX,dddX]=HermiteSpline(P, ind)
[d,n]=size(P);
dttab=1/((n-1)*1000);
t=param(P,ind);
h=t(2:n)-t(1:n-1);
w=bessel(t,P);
for r=1:n-1
    nloc=floor(h(r)/dttab);
    ttabLoc=linspace(0,1,nloc); 
    % Calcolo la curva in forma di Bezier
    Q(:,1)=P(:,r); 
    Q(:,4)=P(:,r+1); 
    Q(:,2)=Q(:,1)+h(r)*w(:,r)/3;
    Q(:,3)=Q(:,4)-h(r)*w(:,r+1)/3;
    [tmp(:,1:nloc,r), dtmp(:,1:nloc,r), ddtmp(:,1:nloc,r), dddtmp(:,1:nloc,r)]=casteljau(Q,ttabLoc);
    X = [X tmp(:,1:nloc,r)];
    dX = [dX dtmp(:,1:nloc,r)./h(r)];
    ddX = [ddX ddtmp(:,1:nloc,r)./(h(r)*h(r))];   
    dddX = [dddX dddtmp(:,1:nloc,r)./(h(r)*h(r)*h(r))];   
end
return
