% function [C, dX, ddX, dddX]=casteljau(b,tab) 
% calcola la curva di Bezier di punti
% di controllo b nei punti del vettore tab

% Input
%    b    punti di controllo dimensione 2 x (n-1) dove 
%         n-1 e' il grado della curva (n punti di controllo)
%    tab  punti di tabulazione
% Output
%    C    curva di Bezier calcolata nei punti tab
%    dX   derivata prima della curva C
%    ddX  derivata seconda della curva C
%    dddX derivata terza della curva C

function [C, dX, ddX, dddX]=casteljau(b,tab)
tab=tab(:);
tab=tab';
[d n]=size(b);
ntab=length(tab);
C = zeros(d,ntab,n);
for l=1:d
    C(l,1:ntab,1:n)=ones(1,ntab)'*b(l,1:n);
end
dX = zeros(d, ntab);
ddX = ones(d, ntab);
for l=1:d
    for r=1:n-1
        for i=1:(n-r)
            C(l,:,i)=(1-tab).*C(l,:,i)+tab.*C(l,:,i+1);
        end
        if (r == n-2)
            dX(l,:) = (n-1)*(C(l,:,2) - C(l,:,1));
        end        
        if (r == n-3)
            ddX(l,:) = (n-1)*(n-2)*(C(l,:,3) - 2*C(l,:,2) + C(l,:,1));
        end        
        if (r == n-4)
            dddX(l,:) = (n-1)*(n-2)*(n-3)*(C(l,:,4) - 3*C(l,:,3) ...
                        + 3*C(l,:,2) - C(l,:,1));
        end
    end
end
if (n == 4),
    for l=1:d,
        dddX(l,1:length(tab)) = diff(diff(diff(b(l,:))))*(n-1)*(n-2)*(n-3);
    end
end
C=C(:,:,1);