% function [X, dX, ddX, dddX] = conica(ttab, ctrl, s) 
% calcola una conica come una Bezier razionale di punti
% di controllo ctrl nei punti del vettore ttab

% Input
%   ctrl punti di controllo dimensione 2 x (n-1) dove 
%        n-1 e' il grado della curva (n punti di controllo)
%   ttab punti di tabulazione
%   s    parametro che determina il tipo di conica
%        0 < s < .5     ellisse 
%         s = .5         parabola
%        .5 < s < 1     iperbole 
% Output
%   X    curva di Bezier razionale calcolata nei punti tab
%   dX   derivata prima della curva X
%   ddX  derivata seconda della curva X
%   dddX derivata terza della curva X

function [X, dX, ddX, dddX] = conica(ttab, ctrl, s)
beta = s / (1-s);
[X, dX, ddX, dddX] = ratbezier(ttab, ctrl, [1 beta 1]');
if beta < 1,
    [X1, dX1, ddX1, dddX1] = ratbezier(ttab, ctrl, [1 -beta 1]');
    X = [X X1]; dX = [dX dX1]; ddX = [ddX ddX1]; dddX = [dddX dddX1];
end
return