% function C=deboor(t,Q,ttab)
% Algoritmo di de Boor per tabulare una curva B-spline di ordine k
%
% t    = vettore dei nodi (n+k componenti)
%
% Q    = punti di controllo de de Boor (dxn)
%
% ttab = vettore dei punti di tabulazione 
%        (crescente e contenuto in [t_(k-1),t_(n+1)])
function C=deboor(t,Q,ttab)
  
  [d,n]=size(Q);
  it=length(t);
  k= it-n;
  ntab=length(ttab);
%
% individuazione dei punti di tabulazione appartenenti ai vari intervalli
%
  ind(k)=1; ind(k+1:n)=zeros(1,n-k);

% ind(r)=indice del primo punto maggiore o uguale a t_r; se ind(r)=0
% non ci sono punti maggiori di t_r
  
  for r=k+1:n   
      if ind(r-1)==0
          break
       end   
    for i=ind(r-1):ntab
       if ttab(i)>= t(r)   
         ind(r)= i;  
         break
       end     
    end    
  end
  ind(n+1)=0;
% 
  for r=k:n
   if ind(r)==0  % 
      break
   end   
     if ind(r+1)~=0
       indl= ind(r+1)-1;
     else
       indl= ntab;
     end     
                                 % ciclo sugli intervalli                       
      tloc=ttab(ind(r):indl);   
      nloc=length(tloc); % punti da tabulare appartenenti all'intervallo t_r; t_(r+1)
      for l=1:d
          Qloc(1:k,1:nloc,l)=Q(l,r-k+1:r)'*ones(1,nloc);
      end
      for j=1:k-1    % struttura analoga a de Casteljau
        clear alfa
        for i=1:k-j
          alfa(i,:) = (tloc-t(i+r-k+j))/(t(i+r)-t(i+r-k+j));
          for l=1:d
              Qloc(i,1:nloc,l)=(1-alfa(i,:)).*Qloc(i,1:nloc,l)+alfa(i,:).*Qloc(i+1,1:nloc,l);
          end
      end
%    
% definizione curva nell'intervallo [t_r,t_(r+1))
%        
    for l=1:d
        C(l,ind(r):indl)=Qloc(1,1:nloc,l);
    end
end   
end
