function varargout = guibezier(varargin)
% GUIBEZIER Application M-file for guibezier.fig
%    FIG = GUIBEZIER launch guibezier GUI.
%    GUIBEZIER('callback_name', ...) invoke the named callback.

% Last Modified by GUIDE v2.0 21-Sep-2002 18:12:09
if nargin == 0  % LAUNCH GUI

	fig = openfig(mfilename,'reuse');

	set(fig, 'Color', get(0,'defaultUicontrolBackgroundColor'));
    set(fig, 'Tag', 'main_figure');

	% Generate a structure of handles to pass to callbacks, and store it. 
	handles = guihandles(fig);
    handles.ttab = [0:.01:1];
    handles.ctrl = [];
    handles.tipo = -1;
    handles.name = '';
    handles.pesi = [];
    handles.ncurves = 0;
    handles.param = 1;
    handles.property = struct('color', 'b', 'trat', 2);
    handles.assi = [0 1 0 1];
    handles.bform = [];
    
    handles.hdata = guibezdata;
	guidata(fig, handles);
    
	if nargout > 0
		varargout{1} = fig;
	end

elseif ischar(varargin{1}) % INVOKE NAMED SUBFUNCTION OR CALLBACK

	try
		if (nargout)
			[varargout{1:nargout}] = feval(varargin{:}); % FEVAL switchyard
		else
			feval(varargin{:}); % FEVAL switchyard
		end
	catch
		disp(lasterr);
	end
end

%| ABOUT CALLBACKS:
%| GUIDE automatically appends subfunction prototypes to this file, and 
%| sets objects' callback properties to call them through the FEVAL 
%| switchyard above. This comment describes that mechanism.
%|
%| Each callback subfunction declaration has the following form:
%| <SUBFUNCTION_NAME>(H, EVENTDATA, HANDLES, VARARGIN)
%|
%| The subfunction name is composed using the object's Tag and the 
%| callback type separated by '_', e.g. 'slider2_Callback',
%| 'figure1_CloseRequestFcn', 'axis1_ButtondownFcn'.
%|
%| H is the callback object's handle (obtained using GCBO).
%|
%| EVENTDATA is empty, but reserved for future use.
%|
%| HANDLES is a structure containing handles of components in GUI using
%| tags as fieldnames, e.g. handles.figure1, handles.slider2. This
%| structure is created at GUI startup using GUIHANDLES and stored in
%| the figure's application data using GUIDATA. A copy of the structure
%| is passed to each callback.  You can store additional information in
%| this structure at GUI startup, and you can change the structure
%| during callbacks.  Call guidata(h, handles) after changing your
%| copy to replace the stored original so that subsequent callbacks see
%| the updates. Type "help guihandles" and "help guidata" for more
%| information.
%|
%| VARARGIN contains any extra arguments you have passed to the
%| callback. Specify the extra arguments by editing the callback
%| property in the inspector. By default, GUIDE sets the property to:
%| <MFILENAME>('<SUBFUNCTION_NAME>', gcbo, [], guidata(gcbo))
%| Add any extra arguments after the last argument, before the final
%| closing parenthesis.

% --------------------------------------------------------------------
function varargout = File_Callback(h, eventdata, handles, varargin)

% --------------------------------------------------------------------
function varargout = Open_Callback(h, eventdata, handles, varargin)
delete_ctrl_pesi(handles);
[ctrl, ttab, pesi, tipo] = loadcurve;
handles.ctrl = ctrl;
handles.ttab = ttab;
handles.pesi = pesi;
handles.tipo = tipo;

[X, dX, ddX, K, T] = calculate(handles.ttab, handles.ctrl, handles.pesi, handles.tipo, handles.param);
handles.X = X;
handles.dX = dX;
handles.ddX = ddX;
handles.K = K;
handles.T = T;
guidata(h,handles);
plotcurve(1, handles.tipo);

visaddpoint(ctrl, pesi);


% --------------------------------------------------------------------
function varargout = Save_Callback(h, eventdata, handles, varargin)

% --------------------------------------------------------------------
function varargout = SaveAs_Callback(h, eventdata, handles, varargin)
uisave;

% --------------------------------------------------------------------
function varargout = Untitled_16_Callback(h, eventdata, handles, varargin)

% --------------------------------------------------------------------
function varargout = AddPoint_Callback(h, eventdata, handles, varargin)
ctrl = handles.ctrl;
pesi = handles.pesi;
[n, d] = size(ctrl);
if (n < 24),
    if (d == 2),
        np = ginput(1);
    else
        np = ctrl(n, :);
    end
    [handles.ctrl, handles.pesi] = addpoint(ctrl, np, pesi);
    [X, dX, ddX, K, T] = calculate(handles.ttab, handles.ctrl, handles.pesi, handles.tipo, handles.param);
    handles.X = X;
    handles.dX = dX;
    handles.ddX = ddX;
    handles.K = K;
    handles.T = T;
    guidata(h, handles);
    clearselectsel;
    plotcurve(1,handles.tipo);
end


% --------------------------------------------------------------------
function varargout = RemovePoint_Callback(h, eventdata, handles, varargin)
ctrl = handles.ctrl;
pesi = handles.pesi;
n = length(ctrl);
if (n > 3),
    handles.ctrl = ctrl(1:n-1,:);
    handles.pesi = pesi(1:n-1);
    [X, dX, ddX, K, T] = calculate(handles.ttab, handles.ctrl, handles.pesi, handles.tipo, handles.param);
    handles.X = X;
    handles.dX = dX;
    handles.ddX = ddX;
    handles.K = K;
    handles.T = T;    
    delete(findobj(handles.hdata, 'Tag', strcat('ETX', num2str(n))));
    delete(findobj(handles.hdata, 'Tag', strcat('ETY', num2str(n))));
    delete(findobj(handles.hdata, 'Tag', strcat('ETZ', num2str(n))));    
    delete(findobj(handles.hdata, 'Tag', strcat('Pesi', num2str(n))));        
    guidata(h, handles);
    clearselectsel;    
    plotcurve(1,handles.tipo);
end

% --------------------------------------------------------------------
function varargout = DegreeElevation_Callback(h, eventdata, handles, varargin)
% Si aggiunge un punto di controllo mantenendo la stessa curva con il metodo
% di degree elevation
ctrl = handles.ctrl;
if (length(ctrl) < 24),
    delete_ctrl_pesi(handles);    
    % Calcolo del nuovo poligono di controllo
    if (handles.tipo==0),
        ctrl = degreeelevation(ctrl')';
        handles.pesi = ones(length(ctrl), 1);
    else        
        [ctrl, pesi] = ratdegreeelevation(ctrl', handles.pesi');
        ctrl = ctrl'; pesi = pesi';
        handles.pesi = pesi;
    end
    handles.ctrl = ctrl;
    guidata(h, handles);
    % Nuovo plot della curva che comunque non e' stata ricalcolta
    clearselectsel;
    plotcurve(1, handles.tipo);
    % Riaggiunta dei campi di edit dei bottoni dei punti di controllo

     visaddpoint(ctrl, pesi);

end


% --------------------------------------------------------------------
function varargout = DegreeReduction_Callback(h, eventdata, handles, varargin)
ctrl = handles.ctrl;
if (length(ctrl) > 3),
    % Eliminazione dei pulsanti dei vecchi punti di controllo
    delete_ctrl_pesi(handles);
    % Calcolo del nuovo poligono di controllo
    ctrl = degreereduction(ctrl')';
    handles.ctrl = ctrl;
    handles.pesi = ones(length(ctrl), 1);
    [X, dX, ddX, K, T] = calculate(handles.ttab, handles.ctrl, handles.pesi, handles.tipo, handles.param);
    handles.X = X;
    handles.dX = dX;
    handles.ddX = ddX;
    handles.K = K;
    handles.T = T;    
    guidata(h, handles);
    % Nuovo plot della curva che comunque non e' stata ricalcolta
    clearselectsel;    
    plotcurve(1, handles.tipo);
    % Riaggiunta dei campi di edit dei bottoni dei punti di controllo
    visaddpoint(ctrl, handles.pesi);
end

% --------------------------------------------------------------------
function varargout = Untitled_10_Callback(h, eventdata, handles, varargin)

% --------------------------------------------------------------------
function varargout = Curve_Callback(h, eventdata, handles, varargin)
plotcurve(1, handles.tipo);

% --------------------------------------------------------------------
function varargout = FirstDerivate_Callback(h, eventdata, handles, varargin)
plotcurve(2, handles.tipo);

% --------------------------------------------------------------------
function varargout = SecondDerivate_Callback(h, eventdata, handles, varargin)
plotcurve(3, handles.tipo);

% --------------------------------------------------------------------
function varargout = Curvature_Callback(h, eventdata, handles, varargin)
plotcurve(4, handles.tipo);

% --------------------------------------------------------------------
function varargout = CurveCurvature_Callback(h, eventdata, handles, varargin)
plotcurve(5, handles.tipo);

% --------------------------------------------------------------------
function varargout = Command_Callback(h, eventdata, handles, varargin)

% --------------------------------------------------------------------
function varargout = Untitled_11_Callback(h, eventdata, handles, varargin)
ctrl = handles.ctrl;
[n, d] = size(ctrl);
if (ctrl(1,:) ~= ctrl(n,:))
    ctrl = handles.ctrl;
    if (length(ctrl) < 24),
        [handles.ctrl, handles.pesi] = addpoint(ctrl, ctrl(1,:), handles.pesi);
        [X, dX, ddX, K, T] = calculate(handles.ttab, handles.ctrl, handles.pesi, handles.tipo, handles.param);
        handles.X = X;
        handles.dX = dX;
        handles.ddX = ddX;
        handles.K = K;
        handles.T = T;        
        guidata(h, handles);
        clearselectsel;        
        plotcurve(1, handles.tipo);
    end
end

function varargout = Exit_Callback(h, eventdata, handles, varargin)
a = questdlg('Are you sure you want to exit?');
switch a
case 'No'
    return
case 'Cancel'
    return
end
MainFigure_DeleteFcn(h, eventdata, handles, varargin);

% --------------------------------------------------------------------
function varargout = PBRBezier_Callback(h, eventdata, handles, varargin)


% --------------------------------------------------------------------
function varargout = JPG_Callback(h, eventdata, handles, varargin)
print -djpeg90 guiplot

% --------------------------------------------------------------------
function varargout = GIF_Callback(h, eventdata, handles, varargin)

% --------------------------------------------------------------------
function varargout = TIFF_Callback(h, eventdata, handles, varargin)
print -depsc -tiff -r200 guiplot


% --------------------------------------------------------------------
function varargout = PNG_Callback(h, eventdata, handles, varargin)
print -dpng guiplot



function rhandles = save_old(handles)
[n, d] = size(handles.ctrl);
% Inserimento del nome della curva
name = inputdlg('Insert the name of the curve', 'Name of the Curve');
if (isempty(name) | strcmp(name, '')),
    warndlg('The name of the curve can''t be empty');
    return
end

if (handles.tipo > -1),
    % Deselezione la curva selezionata attualmente
    clearselectsel;
    % Salvataggio curva corrente
    if handles.name ~= '',
        S = struct('ctrl', handles.ctrl, 'tipo', handles.tipo, ...
            'name', handles.name, 'pesi', handles.pesi, ...
            'X', handles.X, 'dX', handles.dX, 'ddX', handles.ddX, 'K', handles.K, ...
            'T', handles.T, 'ttab', handles.ttab, 'param', handles.param, ...            
            'color', handles.property.color, 'trat', handles.property.trat);
        h_old = findobj(gcf, 'Tag', strcat('des_', handles.name));
        if ~isempty(h_old),
            delete(h_old);
        end
        if (d == 2),
            des = plot(handles.X(:,1), handles.X(:,2), handles.property.color);
        else
            des = plot3(handles.X(:,1), handles.X(:,2), handles.X(:,3), handles.property.color);
        end
        set(des, 'Linewidth', handles.property.trat);
        set(des, 'UserData', S, 'Tag', strcat('des_', handles.name));
    end
end

handles.name = char(name);
if handles.ncurves == 0,
    uimenu(findobj('Tag', 'select_menu'), 'Tag', strcat('cm_', char(name)), 'Label', char(name), 'Callback', strcat('load_save_sel(''', char(name), ''')'), 'Separator', 'on');    
else
    uimenu(findobj('Tag', 'select_menu'), 'Tag', strcat('cm_', char(name)), 'Label', char(name), 'Callback', strcat('load_save_sel(''', char(name), ''')'));
end
handles.ncurves = handles.ncurves + 1;
handles.property.color = 'b';
handles.property.trat = 2;
rhandles = handles;

% --------------------------------------------------------------------
function varargout = NewBezier_Callback(h, eventdata, handles, varargin)
% Verifica compatibilit delle curve
[n, d] = size(handles.ctrl);
if (handles.ncurves > 0) & (d ~= 2),
    warndlg('Le dimensioni delle curve non corrispondono');
    return
end

handles = save_old(handles);

title('Immetti di punti di controllo iniziali');
ctrl = handles.ctrl;
delete_ctrl_pesi(handles);
ctrl = ginput(24);
pesi = ones(length(ctrl), 1);
handles.pesi = pesi;
handles.ctrl = ctrl;
handles.tipo = 0;
handles.ttab = [0:.01:1];
[X, dX, ddX, K, T] = calculate(handles.ttab, handles.ctrl, handles.pesi, 0, handles.param);
handles.X = X;
handles.dX = dX;
handles.ddX = ddX;
handles.K = K;
handles.T = T;
guidata(h, handles);
plotcurve(1, handles.tipo);
visaddpoint(ctrl, pesi)


% --------------------------------------------------------------------
function varargout = New3DBezier_Callback(h, eventdata, handles, varargin)
% Verifica compatibilit delle curve
[n, d] = size(handles.ctrl);
if (handles.ncurves > 0) & (d ~= 3),
    warndlg('Le dimensioni delle curve non corrispondono');
    return
end

handles = save_old(handles);

delete_ctrl_pesi(handles);
ctrl = zeros(5,3);
ctrl(:,1) = [0, .25, .5, .75, 1]';
ctrl(:,2) = [0, .25, .5, .75, 1]';
ctrl(:,3) = [0, .25, .5, .75, 1]';
pesi = ones(length(ctrl), 1);
handles.pesi = pesi;
handles.ctrl = ctrl;
handles.tipo = 0;
handles.ttab = [0:.01:1];
[X, dX, ddX, K, T] = calculate(handles.ttab, handles.ctrl, handles.pesi, 0, handles.param);
handles.X = X;
handles.dX = dX;
handles.ddX = ddX;
handles.K = K;
handles.T = T;
guidata(h, handles);
plotcurve(1, handles.tipo);
visaddpoint(ctrl, pesi);


% --------------------------------------------------------------------
function varargout = NewRationalBezier_Callback(h, eventdata, handles, varargin)
% Verifica compatibilit delle curve
[n, d] = size(handles.ctrl);
if (handles.ncurves > 0) & (d ~= 2),
    warndlg('Le dimensioni delle curve non corrispondono');
    return
end

handles = save_old(handles);

title('Immetti di punti di controllo iniziali');
delete_ctrl_pesi(handles);
ctrl = ginput(24);
pesi = ones(length(ctrl), 1);
handles.pesi = pesi;
handles.ctrl = ctrl;
handles.tipo = 1;
handles.ttab = [0:.01:1];
[X, dX, ddX, K, T] = calculate(handles.ttab, handles.ctrl, handles.pesi, 1, handles.param);
handles.X = X;
handles.dX = dX;
handles.ddX = ddX;
handles.K = K;
handles.T = T;
guidata(h, handles);
plotcurve(1, handles.tipo);
visaddpoint(ctrl, pesi);





% --------------------------------------------------------------------
function varargout = Print_Callback(h, eventdata, handles, varargin)
print



% --------------------------------------------------------------------
function varargout = PrinterSetup_Callback(h, eventdata, handles, varargin)
print -dsetup



% --------------------------------------------------------------------
function varargout = New3DRationalBezier_Callback(h, eventdata, handles, varargin)
% Verifica compatibilit delle curve
[n, d] = size(handles.ctrl);
if (handles.ncurves > 0) & (d ~= 3),
    warndlg('Le dimensioni delle curve non corrispondono');
    return
end

handles = save_old(handles);

delete_ctrl_pesi(handles);
ctrl = zeros(5,3);
ctrl(:,1) = [0, .25, .5, .75, 1]';
ctrl(:,2) = [0, .25, .5, .75, 1]';
ctrl(:,3) = [0, .25, .5, .75, 1]';
pesi = ones(length(ctrl), 1);
handles.pesi = pesi;
handles.ctrl = ctrl;
handles.tipo = 1;
handles.ttab = [0:.01:1];
[X, dX, ddX, K, T] = calculate(handles.ttab, handles.ctrl, handles.pesi, 1, handles.param);
handles.X = X;
handles.dX = dX;
handles.ddX = ddX;
handles.K = K;
handles.T = T;
guidata(h, handles);
plotcurve(1, handles.tipo);
visaddpoint(ctrl, pesi);

% --------------------------------------------------------------------
function varargout = MainFigure_DeleteFcn(h, eventdata, handles, varargin)
delete(handles.hdata);
delete(gcf);


% --------------------------------------------------------------------
function varargout = MainFigure_CreateFcn(h, eventdata, handles, varargin)
% Introduzione musicale
try
    sound(wavread('win.wav'));
catch
    disp('Impossibile eseguire l''introduzione musicale');
end
hold on;
%m = uimenu(gcf, 'Tag', 'select_menu', 'Label', 'Selection');
%uimenu(m, 'Tag', 'deselect_sel', 'Label', 'Deselect', 'Callback', 'deselectsel');



% --------------------------------------------------------------------
function varargout = New_Conica_Callback(h, eventdata, handles, varargin)
% Verifica compatibilit delle curve
[n, d] = size(handles.ctrl);
if (handles.ncurves > 0) & (d ~= 2),
    warndlg('Le dimensioni delle curve non corrispondono');
    return
end

handles = save_old(handles);

title('Immetti di punti di controllo iniziali');

delete_ctrl_pesi(handles);
ctrl = ginput(3);
pesi = [.1 0 0];
handles.pesi = pesi;
handles.ctrl = ctrl;
handles.tipo = 2;
handles.ttab = [0:.01:1];
[X, dX, ddX, K, T] = calculate(handles.ttab, handles.ctrl, handles.pesi, 2, handles.param);
handles.X = X;
handles.dX = dX;
handles.ddX = ddX;
handles.K = K;
handles.T = T;
guidata(h, handles);
plotcurve(1, handles.tipo);
visaddpoint(ctrl, pesi);




% --------------------------------------------------------------------
function varargout = copy_figure_Callback(h, eventdata, handles, varargin)
handles = save_old(handles);
ctrl = handles.ctrl;
delete_ctrl_pesi(handles);

[X, dX, ddX, K, T] = calculate(handles.ttab, handles.ctrl, handles.pesi, handles.tipo, handles.param);
handles.X = X;
handles.dX = dX;
handles.ddX = ddX;
handles.K = K;
handles.T = T;
guidata(h, handles);
plotcurve(1, handles.tipo);

visaddpoint(ctrl, handles.pesi);



% --------------------------------------------------------------------
function varargout = Hermite_Callback(h, eventdata, handles, varargin)
% Verifica compatibilit delle curve
[n, d] = size(handles.ctrl);
if (handles.ncurves > 0) & (d ~= 2),
    warndlg('Le dimensioni delle curve non corrispondono');
    return
end

handles = save_old(handles);

title('Immetti di punti di controllo iniziali');
delete_ctrl_pesi(handles);
ctrl = ginput(24);
pesi = ones(length(ctrl), 1);
handles.pesi = pesi;
handles.ctrl = ctrl;
handles.tipo = 3;
[X, dX, ddX, K, T] = calculate(handles.ttab, handles.ctrl, handles.pesi, 3, handles.param);
handles.X = X;
handles.dX = dX;
handles.ddX = ddX;
handles.K = K;
handles.T = T;
guidata(h, handles);
plotcurve(1, handles.tipo);
visaddpoint(ctrl, pesi);



% --------------------------------------------------------------------
function varargout = Hermite3D_Callback(h, eventdata, handles, varargin)
% Verifica compatibilit delle curve
[n, d] = size(handles.ctrl);
if (handles.ncurves > 0) & (d ~= 3),
    warndlg('Le dimensioni delle curve non corrispondono');
    return
end

handles = save_old(handles);

delete_ctrl_pesi(handles);
ctrl = zeros(5,3);
ctrl(:,1) = [0, .25, .5, .75, 1]';
ctrl(:,2) = [0, .25, .5, .75, 1]';
ctrl(:,3) = [0, .25, .5, .75, 1]';
pesi = ones(length(ctrl), 1);
handles.pesi = pesi;
handles.ctrl = ctrl;
handles.tipo = 3;
[X, dX, ddX, K, T] = calculate(handles.ttab, handles.ctrl, handles.pesi, 3, handles.param);
handles.X = X;
handles.dX = dX;
handles.ddX = ddX;
handles.K = K;
handles.T = T;
guidata(h, handles);
plotcurve(1, handles.tipo);

visaddpoint(ctrl, pesi);





% --------------------------------------------------------------------
function varargout = uniforme_Callback(h, eventdata, handles, varargin)
handles.param = 0;
[X, dX, ddX, K, T] = calculate(handles.ttab, handles.ctrl, handles.pesi, handles.tipo, handles.param);
handles.X = X;
handles.dX = dX;
handles.ddX = ddX;
handles.K = K;
handles.T = T;
guidata(h, handles);
plotcurve(1, handles.tipo);

% --------------------------------------------------------------------
function varargout = cord_length_Callback(h, eventdata, handles, varargin)
handles.param = 1;
[X, dX, ddX, K, T] = calculate(handles.ttab, handles.ctrl, handles.pesi, handles.tipo, handles.param);
handles.X = X;
handles.dX = dX;
handles.ddX = ddX;
handles.K = K;
handles.T = T;
guidata(h, handles);
plotcurve(1, handles.tipo);


% --------------------------------------------------------------------
function varargout = centripeta_Callback(h, eventdata, handles, varargin)
handles.param = 2;
[X, dX, ddX, K, T] = calculate(handles.ttab, handles.ctrl, handles.pesi, handles.tipo, handles.param);
handles.X = X;
handles.dX = dX;
handles.ddX = ddX;
handles.K = K;
handles.T = T;
guidata(h, handles);
plotcurve(1, handles.tipo);

% --------------------------------------------------------------------
function varargout = parameter_Callback(h, eventdata, handles, varargin)



% --------------------------------------------------------------------
function varargout = newton_Callback(h, eventdata, handles, varargin)
% Verifica compatibilit delle curve
[n, d] = size(handles.ctrl);
if (handles.ncurves > 0) & (d ~= 2),
    warndlg('Le dimensioni delle curve non corrispondono');
    return
end

handles = save_old(handles);

title('Immetti di punti di controllo iniziali');

delete_ctrl_pesi(handles);

ctrl = ginput(24);
pesi = ones(length(ctrl), 1);
handles.pesi = pesi;
handles.ctrl = ctrl;
handles.tipo = 4;
handles.ttab = [0:.01:1];
[X, dX, ddX, K, T] = calculate(handles.ttab, handles.ctrl, handles.pesi, 4, handles.param);
handles.X = X;
handles.dX = dX;
handles.ddX = ddX;
handles.K = K;
handles.T = T;
guidata(h, handles);
plotcurve(1, handles.tipo);
visaddpoint(ctrl, pesi);



% --------------------------------------------------------------------
function varargout = newton3D_Callback(h, eventdata, handles, varargin)
% Verifica compatibilit delle curve
[n, d] = size(handles.ctrl);
if (handles.ncurves > 0) & (d ~= 3),
    warndlg('Le dimensioni delle curve non corrispondono');
    return
end

handles = save_old(handles);

delete_ctrl_pesi(handles);

ctrl = zeros(5,3);
ctrl(:,1) = [0, .25, .5, .75, 1]';
ctrl(:,2) = [0, .25, .5, .75, 1]';
ctrl(:,3) = [0, .25, .5, .75, 1]';
pesi = ones(length(ctrl), 1);
handles.pesi = pesi;
handles.ctrl = ctrl;
handles.tipo = 4;
handles.ttab = [0:.01:1];
[X, dX, ddX, K, T] = calculate(handles.ttab, handles.ctrl, handles.pesi, 4, handles.param);
handles.X = X;
handles.dX = dX;
handles.ddX = ddX;
handles.K = K;
handles.T = T;
guidata(h, handles);
plotcurve(1, handles.tipo);
visaddpoint(ctrl, pesi);



% --------------------------------------------------------------------
function varargout = zoom_Callback(h, eventdata, handles, varargin)




% --------------------------------------------------------------------
function varargout = zoom_in_Callback(h, eventdata, handles, varargin)
z = ginput(1);
xmezzi = (handles.assi(1) + handles.assi(2))/2;
if (z(1) < xmezzi),
    handles.assi(2) = xmezzi;
else
    handles.assi(1) = xmezzi;
end
ymezzi = (handles.assi(3) + handles.assi(4))/2;
if (z(2) < ymezzi),
    handles.assi(4) = ymezzi;
else
    handles.assi(3) = ymezzi;
end
axis(handles.assi);
guidata(h, handles);


% --------------------------------------------------------------------
function varargout = zoom_out_Callback(h, eventdata, handles, varargin)
xmezzi = (handles.assi(1) + handles.assi(2))/4;
handles.assi(1) = handles.assi(1) - xmezzi;
handles.assi(2) = handles.assi(2) + xmezzi;
ymezzi = (handles.assi(3) + handles.assi(4))/4;
handles.assi(3) = handles.assi(3) - ymezzi;
handles.assi(4) = handles.assi(4) + ymezzi;
axis(handles.assi);
guidata(h, handles);


% --------------------------------------------------------------------
function varargout = default_Callback(h, eventdata, handles, varargin)
handles.assi = [0 1 0 1];
axis(handles.assi);
guidata(h, handles);



% --------------------------------------------------------------------
function varargout = nuspline_Callback(h, eventdata, handles, varargin)
% Verifica compatibilit delle curve
[n, d] = size(handles.ctrl);
if (handles.ncurves > 0) & (d ~= 2),
    warndlg('Le dimensioni delle curve non corrispondono');
    return
end

handles = save_old(handles);

title('Immetti di punti di controllo iniziali');

delete_ctrl_pesi(handles);

ctrl = ginput(24);
pesi = zeros(length(ctrl), 1);
handles.pesi = pesi;
handles.ctrl = ctrl;
handles.tipo = 5;
[X, dX, ddX, K, T] = calculate(handles.ttab, handles.ctrl, handles.pesi, 5, handles.param);
handles.X = X;
handles.dX = dX;
handles.ddX = ddX;
handles.K = K;
handles.T = T;
guidata(h, handles);
plotcurve(1, handles.tipo);
visaddpoint(ctrl, pesi);



% --------------------------------------------------------------------
function varargout = nuspline3D_Callback(h, eventdata, handles, varargin)
% Verifica compatibilit delle curve
[n, d] = size(handles.ctrl);
if (handles.ncurves > 0) & (d ~= 3),
    warndlg('Le dimensioni delle curve non corrispondono');
    return
end

handles = save_old(handles);

delete_ctrl_pesi(handles);

ctrl = zeros(5,3);
ctrl(:,1) = [0, .25, .5, .75, 1]';
ctrl(:,2) = [0, .25, .5, .75, 1]';
ctrl(:,3) = [0, .25, .5, .75, 1]';
pesi = ones(length(ctrl), 1);
handles.pesi = pesi;
handles.ctrl = ctrl;
handles.tipo = 5;
[X, dX, ddX, K, T] = calculate(handles.ttab, handles.ctrl, handles.pesi, 5, handles.param);
handles.X = X;
handles.dX = dX;
handles.ddX = ddX;
handles.K = K;
handles.T = T;
guidata(h, handles);
plotcurve(1, handles.tipo);
visaddpoint(ctrl, pesi);




% --------------------------------------------------------------------
function varargout = delete_curve_Callback(h, eventdata, handles, varargin)
if (handles.tipo > -1) & (handles.ncurves ~= 0)
    ctrl = handles.ctrl;
    
    delete_ctrl_pesi(handles);    
    
    handles.tipo = -1;
    h_old = findobj(gcf, 'Tag', strcat('des_', handles.name));
    if ~isempty(h_old),
        delete(h_old);
    end
    
    hsel = findobj('Tag', strcat('c_', handles.name));
    if (~(isempty(hsel))),
        for i=1:length(hsel),
            delete(hsel(i));
        end
    end
    hsel = findobj('Tag', strcat('cm_', handles.name));
    if (~(isempty(hsel))),
        for i=1:length(hsel),
            delete(hsel(i));
        end
    end

    handles.ncurves = handles.ncurves - 1;
    guidata(h, handles);
end


function varargout = new_bezier_ctrl(c, h, handles)
handles = save_old(handles);

delete_ctrl_pesi(handles);

ctrl = c';
pesi = ones(length(ctrl), 1);
handles.pesi = pesi;
handles.ctrl = ctrl;
handles.tipo = 0;
handles.ttab = [0:.01:1];
[X, dX, ddX, K, T] = calculate(handles.ttab, handles.ctrl, handles.pesi, 0, handles.param);
handles.X = X;
handles.dX = dX;
handles.ddX = ddX;
handles.K = K;
handles.T = T;
guidata(h, handles);
plotcurve(1, handles.tipo);
visaddpoint(ctrl, pesi);



% --------------------------------------------------------------------
function varargout = subdivision_Callback(h, eventdata, handles, varargin)
% Inserimento del nome della curva
name = inputdlg('Point of Subdivision', 'Subdivision');
sub = str2double(name);
if (isnan(sub)),
    warndlg('You must give me a number');
    return
end
if (sub >= 1) | (sub <= 0),
    warndlg('You must give me a number between 0 and 1');
    return
end

ctrl = handles.ctrl';
delete_curve_Callback(h, eventdata, handles, varargin);
handles = guidata(h);

[csx, cdx] = subdivision(ctrl, sub);
new_bezier_ctrl(csx, h, handles);
change_color('r');
handles = guidata(h);

new_bezier_ctrl(cdx, h, handles);
change_color('k');
handles = guidata(h);

guidata(h, handles);

% --------------------------------------------------------------------
function varargout = torsione_Callback(h, eventdata, handles, varargin)
plotcurve(6, handles.tipo);


% --------------------------------------------------------------------
function varargout = new_bspline_Callback(h, eventdata, handles, varargin)
% Verifica compatibilit delle curve
[n, d] = size(handles.ctrl);
if (handles.ncurves > 0) & (d ~= 2),
    warndlg('Le dimensioni delle curve non corrispondono');
    return
end
[k, a, b, tau, M, OC, ok] = inputbspline;
if (ok == 0),
    return
end
t=knotvett(k,a,b,tau,M,OC);
% la dimensione dello spazio e' k +mu dove mu=sum(M(i))
L=length(tau)+1;
n=k+sum(M);
%l'utente deve inserire gli n+1 punti di controllo
% Se la curva e' chiusa gli ultimi k-1 punti devono essere uguali ai primi
if (OC==0)
    title(strcat('Immetti ', num2str(n) ,' punti di controllo iniziali'));
    nctrl = ginput(n);
    if (length(nctrl) < n),
        warndlg(strcat('I punti di controllo devono essere ', num2str(n)));
        return
    end
else
    %curva chiusa (ne deve dare k-1 di meno)
    title(strcat('Immetti ', num2str(n-k+1) , ' punti di controllo iniziali'));
    nctrl=ginput(n-k+1);
    if (length(nctrl) < n-k+1),
        warndlg(strcat('I punti di controllo devono essere ', num2str(n-k+1)));
        return
    end
    nctrl = [nctrl; nctrl(:,1:k-1)];
end

handles = save_old(handles);

delete_ctrl_pesi(handles);

ttab = ttabbspline(a, b, tau');
handles.ttab = ttab;

handles.pesi = t;
handles.ctrl = nctrl;
handles.tipo = 6;
[X, dX, ddX, K, T] = calculate(handles.ttab, handles.ctrl, handles.pesi, 6, handles.param);
handles.X = X;
handles.dX = dX;
handles.ddX = ddX;
handles.K = K;
handles.T = T;

[tnew,Qnew] = bezierform(k,a,b,tau,M,nctrl',OC);
handles.bform = Qnew';

guidata(h, handles);
plotcurve(1, handles.tipo);
visaddpoint(nctrl, t);

function delete_ctrl_pesi(handles)
ctrl = handles.ctrl;
pesi = handles.pesi;
if (~isempty(ctrl))
    for i=1:length(ctrl)        
        delete(findobj(handles.hdata, 'Tag', strcat('ETX', num2str(i))));
        delete(findobj(handles.hdata, 'Tag', strcat('ETY', num2str(i))));
        delete(findobj(handles.hdata, 'Tag', strcat('ETZ', num2str(i))));
    end
end
if (~isempty(pesi))
    for i=1:length(pesi)
        delete(findobj(handles.hdata, 'Tag', strcat('Pesi', num2str(i))));                
    end
end
return


% --------------------------------------------------------------------
function varargout = knot_insertion_Callback(h, eventdata, handles, varargin)
knot = inputdlg('Inserimento nuovo nodo', 'Nuovo nodo da inserire');
knot = str2double(knot);
if isnan(knot) | isempty(knot),
    warndlg('L''ordine deve essere un valore numerico');
    return
end

delete_ctrl_pesi(handles); 
clearselectsel;
[pesi, ctrl] = knotinsert(handles.pesi, handles.ctrl', knot);
handles.ctrl = ctrl';
handles.pesi = pesi;
[X, dX, ddX, K, T] = calculate(handles.ttab, handles.ctrl, handles.pesi, handles.tipo, handles.param);
handles.X = X;
handles.dX = dX;
handles.ddX = ddX;
handles.K = K;
handles.T = T;
guidata(h, handles);
plotcurve(1, handles.tipo);
visaddpoint(handles.ctrl, handles.pesi);

function varargout = bezier_form_Callback(h, eventdata, handles, varargin)
% warndlg('Funzione non attivata');
if ~isempty(handles.bform),
    bf = findobj('Tag', 'b_form');
    if isempty(bf),
        Qnew = handles.bform';
        hbf = plot(Qnew(1,:),Qnew(2,:),'m',Qnew(1,:),Qnew(2,:),'ms');
        set(hbf, 'Tag', 'b_form');
        set(gcbo, 'Checked', 'on');
        set(gcbo, 'Selected', 'on');        
    else
        delete(bf);
        set(gcbo, 'Checked', 'off');
        set(gcbo, 'Selected', 'off');        
    end
end


% --------------------------------------------------------------------
function varargout = bezier_form2_Callback(h, eventdata, handles, varargin)


delete_ctrl_pesi(handles); 
clearselectsel;
[pesi, ctrl] = bezierform2(handles.pesi, handles.ctrl');
handles.ctrl = ctrl';
handles.pesi = pesi;
[X, dX, ddX, K, T] = calculate(handles.ttab, handles.ctrl, handles.pesi, handles.tipo, handles.param);
handles.X = X;
handles.dX = dX;
handles.ddX = ddX;
handles.K = K;
handles.T = T;
guidata(h, handles);
plotcurve(1, handles.tipo);
visaddpoint(handles.ctrl, handles.pesi);
