% function t=param(P,ind)
% Restituisce i valori del parametro t in cui interpolare i punti 
% contenuti nella matrice P di dimensione d x n
%
% ind indica il tipo di parametrizzazione desiderata
%
% ind=0 uniforme
% ind=1 corda
% ind=2 centripeta
function t=param(P,ind)
[d,n]=size(P);
if ind==0
    % Spaziatura uniforme
    h=1/(n-1);
    t=0:h:1;
elseif ind==1
    % In base alla distanza 
    diff=P(:,2:n)-P(:,1:n-1);
    for i=1:n-1
        h(i)=norm(diff(:,i));
    end
    delta=sum(h);
    h=h/delta;
    t(1)=0;
    for i=2:n
        t(i)=t(i-1)+h(i-1);
    end
else
     % Parametrizzazione Centripeta 
     diff=P(:,2:n)-P(:,1:n-1);
     for i=1:n-1
         h(i)=sqrt(norm(diff(:,i)));
     end
     delta=sum(h);
     h=h/delta;
     t(1)=0;
     for i=2:n
        t(i)=t(i-1)+h(i-1);
    end
 end