% function [X, dX, ddX, dddX] = ratbezier(ttab, ctrl, w) 
% calcola la curva di Bezier razionale di punti
% di controllo ctrl nei punti del vettore tab con pesi w

% Input
%   ctrl punti di controllo dimensione 2 x (n-1) dove 
%        n-1 e' il grado della curva (n punti di controllo)
%   ttab punti di tabulazione
%   w    pesi della curva di Bezier razionale
% Output
%   X    curva di Bezier razionale calcolata nei punti tab
%   dX   derivata prima della curva X
%   ddX  derivata seconda della curva X
%   dddX derivata terza della curva X

function [X, dX, ddX, dddX] = ratbezier(ttab, ctrl, w)
[n d] = size(ctrl);
for i=1:d,
    c(:,i) = w.*ctrl(:,i);
    h(:,i) = w.*ones(n,1);
end
[num, dnum, ddnum, dddnum] = casteljau(c',ttab);
[den, dden, ddden, dddden] = casteljau(h',ttab);
X = num./den;
dX = 1./den .* (dnum - dden.*X);
ddX = 1./den .* (ddnum - 2*dden.*dX - ddden.*X);
if (n>3)
    dddX = 1./den .* (dddnum - dden.*ddX - 3*ddden.*dX - 3*dddden.*X);
end
return
